// close all graphic windows
close all;

//----------------------------------------
//		Preamble
//----------------------------------------
    
// Endogenous variables
    var c_h y_h y_star psi_f s q pi pi_h pi_f pi_star mc_h mc_star a_h a_star rp r_h r_star y_nat_h y_nat_star x_h x_star s_nat_h e e_star
        // observables
        obs_r_h obs_pi_h obs_y_h obs_e obs_q obs_pi obs_pi_f obs_y_star obs_r_star obs_pi_star;

// Exogenous variables
    varexo
        eps_a_h             // home productivity shock
        eps_a_star          // foreign productivity shock
        eps_r_h             // home monetary policy shock
        eps_r_star          // foreign monetary policy shock
        eps_s               // risk premium shock
        me_r_h me_pi_h me_y_h me_e me_q me_r_star me_y_star me_pi_f me_pi me_pi_star;


// Parameters
    parameters beta sigma alpha eta phi theta_h theta_f theta_star h phi_x_h phi_x_star rho_a_h rho_a_star phi_pi_h phi_pi_star
            phi_e_h phi_e_star rho_r_star rho_r_h rho_s
            y_h_SS y_star_SS r_h_SS r_star_SS;

// Calibrations
    h=.9;                           // habit persistence
    beta=0.99;						// temporal discount factor
    sigma=0.5;						// intertemporal elasticity of consumption
    alpha=0.3;						// degree of openness in home economy
    eta=1;							// elasticity of substitution between home and foreign goods
    phi=3;							// labor disutility
    theta_f=0.5;					// import pricing Calvo probability
    theta_h=0.5;					// home producers Calvo probability
    theta_star=0.5;                // foreign producers Calvo probability
    
    // Parameters of productivity shocks
        rho_a_h = .9;
        rho_a_star = .9;

    // Parameters of risk premium shock
        rho_s = 0.9;

    // Monetary policy in the home economy
        rho_r_h = .9;
        phi_pi_h = 2;
        phi_x_h = .4;
        phi_e_h = .4;

    // Monetary policy in the world economy
        phi_pi_star = 2;
        phi_x_star = .4;
        rho_r_star = .9;
        phi_e_star = .4;

    // Steady States
        y_h_SS = 100*(rho_a_h - 1);
        y_star_SS = 100*(rho_a_star - 1);
        r_h_SS = 100*((1/beta) - 1);
        r_star_SS = 100*((1/beta) - 1);

//----------------------------------------
//		Model
//----------------------------------------

model(linear);
    
    # lambda_h = ((1-theta_h)*(1-beta*theta_h)) / theta_h;              // coefficient on marginal cost on domestic phillips curve
    # lambda_f = ((1-theta_f)*(1-beta*theta_f)) / theta_f;              // coefficient on marginal cost on import prices phillips curve
    # lambda_star = ((1-theta_star)*(1-beta*theta_star)) / theta_star;	// coefficient on marginal cost on import prices phillips curve
    # A = (h*sigma*(phi*alpha*eta*(2-alpha)+1)) / (sigma*(phi*alpha*eta*(2-alpha)+1) + (1-h)*((1-alpha)^2)*phi);
    # B = (sigma*(1+phi)) / (sigma*(phi*alpha*eta*(2-alpha)+1) + (1-h)*((1-alpha)^2)*phi);
    # C = ((1+phi)*(1-h)) / (phi*(1-h) + sigma);
    # D = sigma*h / (phi*(1-h) + sigma);

    // HOME ECONOMY
    
        // home inflation
            pi_h = beta*pi_h(+1) + lambda_h*mc_h;

        // imported inflation
            pi_f = beta*pi_f(+1) + lambda_f*psi_f;

        // CPI inflation
            pi = (1-alpha) * pi_h + alpha * pi_f;

        // home marginal cost
            mc_h = phi*y_h - (1+phi)*a_h + alpha*s + (sigma*(c_h - h*c_h(-1)))/(1-h);

        // real exchange rate
            q = psi_f + (1-alpha)*s;

        // law of one price gap
            psi_f - psi_f(-1) = e - e(-1) + pi_star - pi_f;

        // terms of trade definition
            s - s(-1) = pi_f - pi_h;

        // wedge between domestic and world output
            c_h - h*c_h(-1) = y_star - h*y_star(-1) + (1/sigma)*(1-h)*((1-alpha)*s + psi_f);

        // UIP condition
            (r_h - pi(+1)) - (r_star - pi_star(+1)) = q(+1) - q + rp;

        // market clearing condition
            (1-alpha)*c_h = y_h - alpha*eta*(2-alpha)*s + alpha*eta*psi_f - alpha*y_star;

        // home Taylor Rule
            r_h = rho_r_h*r_h(-1) + (1-rho_r_h)*(phi_pi_h*pi + phi_x_h*x_h + phi_e_h*(e-e(-1))) + eps_r_h;

        // home output gap
            
            // definition
                x_h = y_h - y_nat_h;

            // home natural output level
                y_nat_h = ((1+phi)/phi)*(a_h-a_star) - (s_nat_h/phi) + y_nat_star;

            // home natural terms of trade
                s_nat_h = A * s_nat_h(-1) + B * (a_h - h*a_h(-1) - (a_star - h*a_star(-1)));

    // FOREIGN ECONOMY

        // foreign output
            y_star - h*y_star(-1) = y_star(+1) - h*y_star - (1/sigma)*(1-h)*(r_star - pi_star(+1));
        
        // foreign inflation
            pi_star = beta*pi_star(+1) + lambda_star*mc_star;

        // foreign marginal cost
            mc_star = phi*y_star - (1+phi)*a_star - (sigma*(y_star - h*y_star(-1)))/(1-h);

        // foreign Taylor Rule
            r_star = rho_r_star * r_star(-1) + (1-rho_r_star)*(phi_pi_star*pi_star + phi_x_star*x_star + phi_e_star*(e_star-e_star(-1))) + eps_r_star;
   
        // foreign nominal exchange rate
            e_star - e_star(-1) = e(-1) - e;

        // foreign output gap

            // definition of foreign output gap
                x_star = y_star - y_nat_star;
    
            // foreign natural output level
                y_nat_star = C * a_star + D * y_nat_star(-1);

    // EXOGENOUS PROCESSES

        // home technology process
            a_h = rho_a_h*a_h(-1) + eps_a_h;

        // foreign technology process
            a_star = rho_a_star*a_star(-1) + eps_a_star;

        // risk premium process
            rp = rho_s*rp(-1) + eps_s;

    // MEASUREMENT EQUATIONS

        // nominal interest rate
            obs_r_h = r_h_SS + r_h - r_h(-1) + me_r_h;                         
        
        // domestic inflation
            obs_pi_h = pi_h + me_pi_h;

        // home gdp growth
            obs_y_h = y_h_SS + y_h - y_h(-1) + me_y_h;

        // nominal exchange rate change
            obs_e = e - e(-1) + me_e;

        // real exchange rate change
            obs_q = q - q(-1) + me_q;

        // nominal foreign interest rate
            obs_r_star = r_star_SS + r_star - r_star(-1) + me_r_star;

        // foreign gdp growth
            obs_y_star = y_star_SS + y_star - y_star(-1) + me_y_star;

        // import inflation
            obs_pi_f = pi_f + me_pi_f;

        // SOE CPI inflation
            obs_pi = pi + me_pi;

        // foreign CPI inflation
            obs_pi_star = pi_star + me_pi_star;

end;

//----------------------------------------
//		Initial Values
//----------------------------------------

initval;

obs_r_h = r_h_SS;
obs_r_star = r_star_SS;
obs_y_h = y_h_SS;
obs_y_star = y_star_SS;

end;


steady;

//----------------------------------------
//		Check Blanchard-Kahn conditions
//----------------------------------------

check;

//----------------------------------------
//		Shocks
//----------------------------------------

shocks;

var eps_a_h; stderr 1;
var eps_a_star; stderr 1;
var eps_r_h; stderr 1;
var eps_r_star; stderr 1;
var eps_s; stderr 1;

// measurement errors
var me_r_h; stderr 1;
var me_pi_h; stderr 1;
var me_y_h; stderr 1;
var me_e; stderr 1;
var me_q; stderr 1;
var me_r_star; stderr 1;
var me_y_star; stderr 1;
var me_pi_f; stderr 1;
var me_pi; stderr 1;
var me_pi_star; stderr 1;

end;

//----------------------------------------
//		Observed Variables
//----------------------------------------

varobs obs_r_h obs_r_star obs_q obs_y_h obs_y_star obs_pi_h obs_pi_f obs_pi obs_pi_star obs_e;
                
//----------------------------------------
//		Initialize Estimated Variables
//----------------------------------------

//estimated_params_init;
//end;

//----------------------------------------
//		Priors
//----------------------------------------

estimated_params;
//  parameter name      initival    lb          ub          prior shape     param1      param2      param3      param4      jscale
    //alpha,                                                  beta_pdf,       0.5,        0.25;
    h,                                                      beta_pdf,       0.5,        0.25;
    sigma,                                                  gamma_pdf,      1.2,        0.4;   
    phi,                                                    gamma_pdf,      1.5,        0.75;
    eta,                                                    gamma_pdf,      1.5,        0.75;
    theta_h,                                                gamma_pdf,      0.5,        0.25;
    theta_f,                                                gamma_pdf,      0.5,        0.25;
    theta_star,                                             gamma_pdf,      0.5,        0.25;

    phi_x_h,                                                gamma_pdf,      0.25,       0.13;
    phi_pi_h,                                               gamma_pdf,      1.5,        0.3;
    phi_e_h,                                                gamma_pdf,      0.25,       0.13;
    rho_r_h,                                                beta_pdf,       0.5,        0.25;
    rho_r_star,                                             beta_pdf,       0.5,        0.25;
    rho_a_h,                                                beta_pdf,       0.5,        0.25;
    rho_a_star,                                             beta_pdf,       0.5,        0.25;
    phi_x_star,                                             gamma_pdf,      0.25,       0.13;
    phi_pi_star,                                            gamma_pdf,      1.5,        0.3;
    phi_e_star,                                             gamma_pdf,      0.25,       0.13;

    stderr eps_a_h,                                         inv_gamma_pdf,  1,        4;
    stderr eps_a_star,                                      inv_gamma_pdf,  1,        4;
    stderr eps_r_h,                                         inv_gamma_pdf,  1,        4;
    stderr eps_r_star,                                      inv_gamma_pdf,  1,        4;                        
    stderr eps_s,                                           inv_gamma_pdf,  1,        4;

end;


//----------------------------------------
//		Computation
//----------------------------------------

stoch_simul(irf=25, nograph);

//----------------------------------------
//		Estimation
//----------------------------------------

estimation(datafile=mymodeldata8, conf_sig = 0.95, mh_replic=20000, mh_jscale=0.35,
            mh_nblocks=5, mh_drop=0.5, mode_check, mode_compute=4, bayesian_irf, smoother, tex)
            obs_r_h obs_pi_h obs_y_h obs_e obs_q obs_pi obs_pi_f obs_y_star obs_r_star obs_pi_star;

//----------------------------------------
//		Sensitivity Analysis
//----------------------------------------

//dynare_sensitivity(datafile=mymodeldata8, ppost=1, rmse=1, mode_file=mymodel3_mh_mode);

//----------------------------------------
//		Model Comparison
//----------------------------------------

//model_comparison(marginal_density=laplace) mymodel mymodel2;
